// Performance.c
// Runs on the TM4C1294 microcontroller
// Use the SysTick timer to measure approximately how long it takes to
// calculate a square root.  Also, toggle PG2 to use an oscilloscope
// or logic analyzer to measure the execution time as the high pulse
// time of the LED.
// Daniel Valvano and Jonathan Valvano
// September 10, 2014

/* This example accompanies the book
   "Embedded Systems: Real Time Interfacing to ARM Cortex M Microcontrollers",
   ISBN: 978-1463590154, Jonathan Valvano, copyright (c) 2014
   Program 3.17

 Copyright 2014 by Jonathan W. Valvano, valvano@mail.utexas.edu
    You may use, edit, run or distribute this file
    as long as the above copyright notice remains
 THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 For more information about my classes, my research, and my books, see
 http://users.ece.utexas.edu/~valvano/
 */

// oscilloscope connected to PN1

#include <stdint.h>
#include "..//tm4c1294ncpdt.h"
#include "PLL.h"
#include <math.h>

#define PN0            (*((volatile uint32_t *)0x40064004))
#define PN1            (*((volatile uint32_t *)0x40064008))
#define PN2            (*((volatile uint32_t *)0x40064010))
#define PN3            (*((volatile uint32_t *)0x40064020))

void DisableInterrupts(void); // Disable interrupts
void EnableInterrupts(void);  // Enable interrupts

volatile uint32_t before, elapsed, ss, tt;
uint32_t offset;
// Newton's method (this one not tested, one in math.h is tested
// s is an integer
// sqrt(s) is an integer
/*unsigned long sqrt(unsigned long s){
unsigned long t;         // t*t will become s
int n;                   // loop counter to make sure it stops running
  t = s/16+1;            // initial guess 
  for(n = 16; n; --n){   // guaranteed to finish
    t = ((t*t+s)/t)/2;  
  }
  return t; 
}*/

// Initialize SysTick with busy wait running at bus clock.
void SysTick_Init(void){
  NVIC_ST_CTRL_R = 0;               // disable SysTick during setup
  NVIC_ST_RELOAD_R = 0x00FFFFFF;    // maximum reload value
  NVIC_ST_CURRENT_R = 0;            // any write to current clears it
  NVIC_ST_CTRL_R = 0x05;            // enable SysTick with core clock
  
}
//test code
int main(void){ 
  PLL_Init(); // 120 MHz
                                   // activate clock for Port N
  SYSCTL_RCGCGPIO_R |= SYSCTL_RCGCGPIO_R12;
                                   // allow time for clock to stabilize
  while((SYSCTL_PRGPIO_R&SYSCTL_PRGPIO_R12) == 0){};
  GPIO_PORTN_DIR_R |= 0x0F;        // make PN3-0 out (PN1-0 built-in LED1-2)
  GPIO_PORTN_AFSEL_R &= ~0x0F;     // disable alt funct on PN3-0
  GPIO_PORTN_DEN_R |= 0x0F;        // enable digital I/O on PN3-0
                                   // configure PN3-0 as GPIO
  GPIO_PORTN_PCTL_R = (GPIO_PORTN_PCTL_R&0xFFFF0000)+0x00000000;
  GPIO_PORTN_AMSEL_R &= ~0x0F;     // disable analog functionality on PN3-0


  SysTick_Init(); // initialize SysTick timer, see SysTick.c
  before = NVIC_ST_CURRENT_R;
  // nothing
  offset = (before-NVIC_ST_CURRENT_R)&0x00FFFFFF;
  ss = 100;
  before = NVIC_ST_CURRENT_R;
  tt = sqrt(ss);
  elapsed = ((before-NVIC_ST_CURRENT_R)&0x00FFFFFF)-offset;
  while(1){
    PN1 = 0x02;                 // turn on LED
    tt = sqrt(ss);
    PN1 = 0x00;                 // turn off LED
    // anything can go here, after the LED goes off before the repeat
    // use the oscilloscope/logic analyzer to measure high pulse time
    // this can be easier to see if there is also some low pulse time
    tt = sqrt(ss);
  }
}
